import processing.serial.*;

import cc.arduino.Arduino;
import controlP5.*;

ControlP5 controlP5;

int nDigitalPins = 14;
int nAnalogPins = 6;
int ledPin = 13;
DigitalPin[] digitalPins = new DigitalPin[nDigitalPins];
AnalogPin[] analogPins = new AnalogPin[nAnalogPins];
int INPUT_MODE = 0;
int INPUT_PULLUP_MODE = 1;
int OUTPUT_MODE = 2;
int PWM_MODE = 3;
boolean ledOn = false;
Textlabel[] legend = new Textlabel[8];
Textlabel statusMessage;
Textlabel portChoiceMessage;
String ports[];
String selectedPort;
Arduino arduino;
boolean connected = false;
int hoverDigitalPin = -1;
int hoverAnalogPin = -1;
boolean removeList = false;

int originX = 47;
int originY = 111;
int legendX = originX - 30;
int legendY = originY - 73;
int titleX = 8;
int titleY = 4;
int logoX = 270;
int logoY = 470;
int xLED = originX + 203;
int yLED = originY + 63;
int inputPinColor = color(0xf5, 0xc6, 0x00);
int inputPinDarkColor = color(0x92, 0x76, 0x00);
int outputPinColor = color(0xff, 0x0, 0x0);
int outputPinDarkColor = color(0x93, 0x00, 0x00);
int boardBlue = color(0x0b, 0x7a, 0xb1);
int boardDarkBlue = color(0x00, 0x38, 0x6a);
int gray = color(128);
int pinSize = 13;

PImage backgroundImage;
PImage titleImage;
PImage logoImage;

MultiList activeList = null;
MultiList portList = null;

void setup() {
    size(550, 500);
    frameRate(12);
    background(0);
    backgroundImage = loadImage("ArduinoDuemilanoveBlue.png");
    titleImage = loadImage("toolduinoLogo.png");
    logoImage = loadImage("logo.png");
    controlP5 = new ControlP5(this);


    int x = originX + 410;
    int y = originY + 7;
    int delta = -15;

    for (int i = 0; i < nDigitalPins; i++) {
        digitalPins[i] = new DigitalPin(i);
        digitalPins[i].x = x + (i * delta);
        if (i == 7) {
            x -= 7;
        }
        digitalPins[i].y = y;
        if ((i == 3) || (i == 5) || (i == 6) || (i == 9) || (i == 10) || (i == 11)) {
            digitalPins[i].isPWM = true;
        }
    }

    x = originX + 337;
    y = originY + 294;
    delta = 15;
    for (int i = 0; i < nAnalogPins; i++) {
        analogPins[i] = new AnalogPin();
        analogPins[i].x = x + (i * delta);
        analogPins[i].y = y;
    }
    
    
    // Legend
    legend[0] = new Textlabel(this, "RIGHT-CLICK DIGITAL PIN TO SET PIN MODE", legendX , legendY, 200, 20, color(255), ControlP5.standard56);
    legend[1] = new Textlabel(this, "OUTPUT:", legendX , legendY+16, 200, 20, outputPinColor, ControlP5.standard56);
    legend[2] = new Textlabel(this, "=LOW    =HIGH", legendX+110 , legendY+16, 200, 20, outputPinColor, ControlP5.standard56);
    legend[3] = new Textlabel(this, "INPUT:", legendX, legendY+32, 200, 20, inputPinColor, ControlP5.standard56);
    legend[4] = new Textlabel(this, "=LOW    =HIGH", legendX+110 , legendY+32, 200, 20, inputPinColor, ControlP5.standard56);
    legend[5] = new Textlabel(this, "INPUT W/ PULLUP:", legendX, legendY+48, 200, 20, inputPinColor, ControlP5.standard56);
    legend[6] = new Textlabel(this, "=LOW    =HIGH", legendX+110 , legendY+48, 200, 20, inputPinColor, ControlP5.standard56);
    legend[7] = new Textlabel(this, "CLICK ANALOG PIN TO READ VALUE", legendX+120 , originY+322, 200, 20, color(255), ControlP5.standard56);
    portChoiceMessage = new Textlabel(this, "CHOOSE ARDUINO CONNECTION PORT:", legendX, legendY, 200, 20, color(255), ControlP5.standard56);


}

void mouseReleased() {
    if ((activeList != null) && (removeList)) {
        activeList.remove();
	removeList = false;
	activeList = null;
    }
}

void mouseMoved() {
    int pinSelected = getDigitalPinSelected(mouseX, mouseY);
    if (pinSelected >= 0) {
	hoverDigitalPin = pinSelected;
	return;
    } else {
	pinSelected = getAnalogPinSelected(mouseX, mouseY);
	if (pinSelected >= 0) {
	    hoverAnalogPin = pinSelected;
	    return;
	}
    }
    hoverDigitalPin = -1;
    hoverAnalogPin = -1;
}

void mousePressed() {
    int pinSelected = getDigitalPinSelected(mouseX, mouseY);
    if (pinSelected >= 0) {
        if (activeList != null) {
            activeList.remove();
        }
        DigitalPin p = digitalPins[pinSelected];
        if (mouseButton == LEFT) {
            digitalPins[pinSelected].click();
        }
        if (mouseButton == RIGHT) {
            int y;
            if (p.isPWM) {
                y = p.y - 47;
            } else {
                y = p.y - 36;
            }
            MultiList l = controlP5.addMultiList("list" + pinSelected, p.x, y, 88, 11);
            l.setId(pinSelected);
            MultiListButton b;
            b = l.add("OUTPUT", OUTPUT_MODE);
            b.captionLabel().setFont(ControlP5.standard56);
            b.setColorBackground(boardDarkBlue);
            b.setColorActive(boardBlue);
            b.setColorLabel(outputPinColor);
            b = l.add("INPUT", INPUT_MODE);
            b.captionLabel().setFont(ControlP5.standard56);
            b.setColorBackground(boardDarkBlue);
            b.setColorActive(boardBlue);
            b.setColorLabel(inputPinColor);
            b = l.add("INPUT W/ PULLUP", INPUT_PULLUP_MODE);
            b.captionLabel().setFont(ControlP5.standard56);
            b.setColorBackground(boardDarkBlue);
            b.setColorActive(boardBlue);
            b.setColorLabel(inputPinColor);
            if (p.isPWM) {
                b = l.add("PWM", PWM_MODE);
                b.captionLabel().setFont(ControlP5.standard56);
                b.setColorBackground(boardDarkBlue);
                b.setColorActive(boardBlue);
            }
            activeList = l;
        }
        return;
    }
    pinSelected = getAnalogPinSelected(mouseX, mouseY);
    if (pinSelected >= 0) {
        if (mouseButton == LEFT) {
            analogPins[pinSelected].click();
            if (analogPins[pinSelected].active) {
                addAnalogSlider(pinSelected);
            } else {
                removeAnalogSlider(pinSelected);
            }
        }
    }
}

void draw() {

    background(0);
    image(titleImage, titleX, titleY);
    image(logoImage, logoX, logoY);

    if (!connected) {
	portChoiceMessage.draw(this);
	if (statusMessage != null) {
	    statusMessage.draw(this);
	}

	if (portList == null) {
	    ports = Arduino.list();
	    // Create the port list.
            MultiListButton b;

	    // HACK: determine width of list
	    int maxWidth = 0;
	    BitFontRenderer bfr = new BitFontRenderer();
	    for(int i=0;i<ports.length;i++) {
		controlP5.Label l = new controlP5.Label(this, ports[i], 20, 11);
		l.setFont(ControlP5.standard56);
		int w = bfr.getWidth(ports[i], l);
		maxWidth = max(maxWidth, w);
	    }

	    portList = controlP5.addMultiList("portList", legendX, legendY+16, maxWidth + 10, 11);
	    for(int i=0;i<ports.length;i++) {
		b = portList.add(ports[i], i);
		b.captionLabel().setFont(ControlP5.standard56);
		b.setColorBackground(boardDarkBlue);
		b.setColorActive(boardBlue);
		b.setColorLabel(color(255));
	    }
	}

	if (selectedPort != null) {
	    try {
		arduino = new Arduino(this, selectedPort);
		connected = true;
		statusMessage = null;
	    } catch (Exception ex) {
		connected = false;
		statusMessage = new Textlabel(this, "CONNECTION ERROR FOR " + selectedPort.toUpperCase() + " : " + ex.getMessage(), legendX, legendY+32+(ports.length*11), 500, 20, color(255, 0, 0), ControlP5.standard56);
		selectedPort = null;
	    }
	}

	return;
    } else {
	if (portList != null) {
	    portList.remove();
	}
    }


    image(backgroundImage, originX, originY);

    readInput();
    writeOutput();

    for(int i=0;i<legend.length;i++) {
        legend[i].draw(this);
    }
    strokeWeight(1);
    noFill();
    stroke(outputPinColor);
    rect(legendX+94, legendY+14, pinSize, pinSize);
    fill(outputPinColor);
    rect(legendX+139, legendY+14, pinSize, pinSize);

    noFill();
    stroke(inputPinColor);
    rect(legendX+94, legendY+30, pinSize, pinSize);
    fill(inputPinColor);
    rect(legendX+139, legendY+30, pinSize, pinSize);

    noFill();
    stroke(inputPinColor);
    rect(legendX+94, legendY+46, pinSize, pinSize);
    int mid = pinSize/2;
    int xx = legendX+94;
    int yy = legendY+46;

    point(xx+mid, yy+2);
    point(xx+mid-1, yy+3);
    point(xx+mid, yy+3);
    point(xx+mid+1, yy+3);
    point(xx+mid-2, yy+4);
    point(xx+mid-1, yy+4);
    point(xx+mid, yy+4);
    point(xx+mid+1, yy+4);
    point(xx+mid+2, yy+4);

    fill(inputPinColor);
    rect(legendX+139, legendY+46, pinSize, pinSize);
    stroke(0);
    xx = legendX+139;
    point(xx+mid, yy+2);
    point(xx+mid-1, yy+3);
    point(xx+mid, yy+3);
    point(xx+mid+1, yy+3);
    point(xx+mid-2, yy+4);
    point(xx+mid-1, yy+4);
    point(xx+mid, yy+4);
    point(xx+mid+1, yy+4);
    point(xx+mid+2, yy+4);
    
    int pinColor;

    // Digital pins
    DigitalPin dp;
    for (int i = 2; i < nDigitalPins; i++) {
        dp = digitalPins[i];
        pinColor = gray;
        if ((dp.mode == INPUT_MODE) || (dp.mode == INPUT_PULLUP_MODE)) {
            pinColor = inputPinColor;
        } else {
            if ((dp.mode == OUTPUT_MODE) || (dp.mode == PWM_MODE)) {
                pinColor = outputPinColor;
            }
        }
        stroke(pinColor);
        if (dp.digitalSignal == Arduino.HIGH) {
            fill(pinColor);
        } else {
	    fill(0);
	}

        rect(dp.x, dp.y, pinSize, pinSize);

	if (i == hoverDigitalPin) {
	    stroke(255);
	    rect(dp.x, dp.y, pinSize, pinSize);
	}

	if (dp.mode == INPUT_PULLUP_MODE) {
	    if (dp.digitalSignal == Arduino.HIGH) {
		stroke(0);
	    } else {
		stroke(inputPinColor);
	    }
	    point(dp.x+mid, dp.y+2);
	    point(dp.x+mid-1, dp.y+3);
	    point(dp.x+mid, dp.y+3);
	    point(dp.x+mid+1, dp.y+3);

	    point(dp.x+mid-2, dp.y+4);
	    point(dp.x+mid-1, dp.y+4);
	    point(dp.x+mid, dp.y+4);
	    point(dp.x+mid+1, dp.y+4);
	    point(dp.x+mid+2, dp.y+4);

	}

        if (dp.mode == PWM_MODE) {
            stroke(pinColor);
            strokeWeight(2);
            line(dp.x + (pinSize / 2) + 1, dp.y + pinSize / 2, dp.x + (pinSize / 2) + 1, dp.y - 20);
            strokeWeight(1);
            fill(pinColor);
            rect(dp.x + pinSize / 2 - 1, dp.y + pinSize / 2 - 1, 3, 3);
            noFill();
        }

    }

    // Analog pins
    AnalogPin ap;
    for (int i = 0; i < nAnalogPins; i++) {
        ap = analogPins[i];
        fill(0);
        if (ap.active) {
            stroke(inputPinColor);
        } else {
            stroke(gray);
        }
        rect(ap.x, ap.y, pinSize, pinSize);

	if (i == hoverAnalogPin) {
	    stroke(255);
	    rect(ap.x, ap.y, pinSize, pinSize);
	}

        if (ap.active) {
            stroke(inputPinColor);
            strokeWeight(2);
            line(ap.x + (pinSize / 2) + 1, ap.y + pinSize / 2, ap.x + (pinSize / 2) + 1, ap.y + 20);
            strokeWeight(1);
            fill(inputPinColor);
            rect(ap.x + pinSize / 2 - 1, ap.y + pinSize / 2 - 1, 3, 3);
            fill(0);
        }
    }

    // Light the LED if needed
    if (ledOn) {
        fill(0xff, 0xdf, 0x5e);
    } else {
        fill(0x8f, 0x6d, 0x40);
    }
    noStroke();
    rect(xLED, yLED, 10, 7);
    noFill();

}

void controlEvent(ControlEvent e) {
    if (e.controller().name().startsWith("list")) {
        int i = e.controller().id();
        if (i >= 0) {
            DigitalPin p = digitalPins[i];
            int value = (int) e.controller().value();
            if ((p.mode != PWM_MODE) && (value == PWM_MODE)) {
                addPWMSlider(i);
            } else {
                if ((p.mode == PWM_MODE) && (value != PWM_MODE)) {
                    removePWMSlider(i);
                }
            }
            p.setMode(value);
	    removeList = true;
        }
    }
    if (e.controller().name().startsWith("pwmSlider")) {
        int i = e.controller().id();
        int pin = i - nDigitalPins;
        DigitalPin p = digitalPins[pin];
        int value = (int) e.controller().value();
        p.setPWMSignal(value);
    }

    if (e.controller().name().equals("portList")) {
	selectedPort = ports[(int)e.controller().value()];
	stroke(0);
	fill(0);
	rect(legendX, legendY+32+(ports.length*11), 500, 20);
	statusMessage = new Textlabel(this, "CONNECTING TO " + selectedPort.toUpperCase() + " .....", legendX, legendY+32+(ports.length*11), 250, 20, color(255), ControlP5.standard56);
	statusMessage.draw(this);
    }
}

void addPWMSlider(int pin) {
    DigitalPin p = digitalPins[pin];
    Slider s = controlP5.addSlider("pwmSlider" + pin, 0, 255, p.x + (pinSize / 4) + 1, p.y - 60, 6, 50);
    s.setColorBackground(outputPinDarkColor);
    s.setColorForeground(outputPinColor);
    s.setColorActive(outputPinColor);
    s.setId(nDigitalPins + pin);
    s.setCaptionLabel("");
    s.setDecimalPrecision(0);
}

void removePWMSlider(int pin) {
    Controller c = controlP5.controller("pwmSlider" + pin);
    if (c != null) {
        c.remove();
    }

}

void addAnalogSlider(int pin) {
    AnalogPin p = analogPins[pin];
    Slider s = controlP5.addSlider("analogSlider" + pin, 0, 1023, p.x + (pinSize / 4) + 1, p.y + 20, 6, 50);
    s.setColorBackground(inputPinDarkColor);
    s.setColorForeground(inputPinColor);
    s.setColorActive(inputPinColor);
    s.setCaptionLabel("");
    s.setDecimalPrecision(0);
}

void removeAnalogSlider(int pin) {
    Controller c = controlP5.controller("analogSlider" + pin);
    if (c != null) {
        c.remove();
    }

}

int getDigitalPinSelected(int x, int y) {
    for (int i = 0; i < nDigitalPins; i++) {
        DigitalPin p = digitalPins[i];
        if ((x > p.x) && (x < p.x + pinSize) && (y > p.y) && (y < p.y + pinSize)) {
            return i;
        }
    }
    return -1;
}

int getAnalogPinSelected(int x, int y) {
    for (int i = 0; i < nAnalogPins; i++) {
        AnalogPin p = analogPins[i];
        if ((x > p.x) && (x < p.x + pinSize) && (y > p.y) && (y < p.y + pinSize)) {
            return i;
        }
    }
    return -1;
}

void readInput() {
    for (int i = 0; i < nAnalogPins; i++) {
        if (analogPins[i].active) {
            if (arduino != null) {
                int value = arduino.analogRead(i);
                Slider s = (Slider)controlP5.controller("analogSlider" + i);
                if (s != null) {
                    s.setValue(value);
                }
            }
        }
    }
    for (int i = 0; i < nDigitalPins; i++) {
        if ((digitalPins[i].mode == INPUT_MODE) || (digitalPins[i].mode == INPUT_PULLUP_MODE)) {
            if (arduino != null) {
                digitalPins[i].digitalSignal = arduino.digitalRead(i);
            }
            if (i == ledPin) {
                if (digitalPins[i].digitalSignal == Arduino.HIGH) {
                    ledOn = true;
                } else {
                    ledOn = false;
                }
            }
        }
    }

}

void writeOutput() {
    for (int i = 0; i < nDigitalPins; i++) {
        if (!digitalPins[i].shouldWrite) {
            continue;
        } else {
            digitalPins[i].shouldWrite = false;
        }
        if (digitalPins[i].mode == OUTPUT_MODE) {
            if (arduino != null) {
               arduino.digitalWrite(i, digitalPins[i].digitalSignal);
            }
            if (i == ledPin) {
                if (digitalPins[i].digitalSignal == Arduino.HIGH) {
                    ledOn = true;
                } else {
                    ledOn = false;
                }
            }
        }
        if (digitalPins[i].mode == PWM_MODE) {
            if (arduino != null) {
                arduino.analogWrite(i, digitalPins[i].pwmSignal);
            }
        }
    }
}
